#!/usr/bin/python
"""Run an executable on a platform

Usage:
    platformrun [-v | -vv] [options] PLATFORM EXECUTABLE
    platformrun -h

Options:
    -h --help           Show this usage message
    -c --config CONF    Specify the measurement configuration to load
                            [default: ~/.measurementrc]
    -t --tools CONF     Config file for the tools needed to run on a platform
                            [default: ~/.platformrunrc]
    -v --verbose        Be verbose

    PLATFORM        Specify the platform on which to run.
                    Available platforms are:
                        stm32f0discovery
                        atmega328p
                        msp-exp430f5529


"""
from docopt import docopt

import subprocess, os, os.path
import threading
from collections import namedtuple
import tempfile
import json

import pyenergy
from time import sleep
import logging

logger = logging.getLogger(__name__)
warning = logger.warning
debug = logger.debug


tool_config = None
measurement_config = None

#######################################################################

def gdb_launch(gdbname, port, fname):
    if logger.getEffectiveLevel() == logging.DEBUG:
        silence = "-batch"
    else:
        silence = "-batch-silent"

    cmdline = '{gdbname} {silence} -ex "set confirm off" \
                  -ex "tar ext :{port}" \
                  -ex "monitor reset halt" \
                  -ex "load" \
                  -ex "delete breakpoints" \
                  -ex "break exit" \
                  -ex "break _exit" \
                  -ex "continue" \
                  -ex "quit" \
                  {fname}'.format(**locals())
    os.system(cmdline)

def background_proc(cmd, stdout=None, stderr=None):

    def run_proc(ev):
        debug("Starting background proc: \"{}\"".format(cmd))
        if logger.getEffectiveLevel() == logging.DEBUG:
            p = subprocess.Popen(cmd, shell=True)
        else:
            p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        while not ev.isSet():
            if logger.getEffectiveLevel() == logging.DEBUG:
                out = p.stdout.read(1)
                err = p.stderr.read(1)
            else:
                out = err = ''
            if (out == '' or err == '') and p.poll() != None:
                break
            ev.wait(0.1)
        debug("Killing background proc: \"{}\"".format(cmd))
        p.kill()

    ev = threading.Event()
    t = threading.Thread(target=run_proc, args=(ev,))
    t.start()

    return ev

def kill_background_proc(p):
    p.set()

def foreground_proc(cmd):
    debug("Starting foreground proc: \"{}\"".format(cmd))
    if logger.getEffectiveLevel() == logging.DEBUG:
        p = subprocess.Popen(cmd, shell=True)
    else:
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    while True:
        if logger.getEffectiveLevel() == logging.DEBUG and p.stdout:
            out = p.stdout.read(1)
        else:
            out = ''

        if logger.getEffectiveLevel() == logging.DEBUG and p.stderr:
            err = p.stderr.read(1)
        else:
            err =''
        if (out == '' or err == '') and p.poll() != None:
            break


def setupMeasurement(platform):
    # First check that all tools for platform are available
    if platform not in tool_config['enabled']:
        raise RuntimeError("Platform {0} does not have all the tools configured. Please rerun configure with --enable-{0}".format(platform))

    em = pyenergy.EnergyMonitor(measurement_config[platform]['energy-monitor'])
    mp = int(measurement_config[platform]['measurement-point'])

    em.connect()
    em.enableMeasurementPoint(mp)
    em.clearNumberOfRuns(mp)
    em.measurement_params[mp]['resistor'] = int(measurement_config[platform]['resistor'])
    em.setTrigger(measurement_config[platform]['trigger-pin'], mp)

    return em

def finishMeasurement(platform, em):
    mp = int(measurement_config[platform]['measurement-point'])

    while not em.measurementCompleted(mp):
        sleep(0.1)
    m = em.getMeasurement(mp)

    em.disconnect()
    return m

# Display units nicer
def prettyPrint(v):
    units = ['', 'm', 'u', 'n', 'p']

    for unit in units:
        if v > 1.0:
            return "{: >8.3f} {}".format(v, unit)
        v *= 1000.
    return "{}".format(v)

#######################################################################

def loadConfiguration(fname):
    global measurement_config

    fname = os.path.expanduser(fname)
    measurement_config = json.load(open(fname))

def loadToolConfiguration(fname):
    global tool_config

    fname = os.path.expanduser(fname)
    tool_config = json.load(open(fname))

#######################################################################

def stm32f0discovery(fname):
    em = setupMeasurement("stm32f0discovery")

    stproc = background_proc(tool_config['tools']['stutil'] + " -p 2001 -c 0x0bb11477 -v0")
    gdb_launch(tool_config['tools']['arm_gdb'], 2001, fname)
    kill_background_proc(stproc)

    return finishMeasurement("stm32f0discovery", em)


def stm32vldiscovery(fname):
    em = setupMeasurement("stm32vldiscovery")

    stproc = background_proc(tool_config['tools']['stutil'] + " -p 2002 -c 0x1ba01477 -v0")
    gdb_launch(tool_config['tools']['arm_gdb'], 2002, fname)
    kill_background_proc(stproc)

    return finishMeasurement("stm32vldiscovery", em)


def atmega328p(fname):
    em = setupMeasurement("atmega328p")

    # Create temporary file and convert to hex file
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.close()
    foreground_proc("{} -O ihex {} {}".format(tool_config['tools']['avr_objcopy'], fname, tf.name))

    # Flash the hex file to the AVR chip
    ser_id = measurement_config['atmega328p']['serial-dev-id']
    cmdline = "{} -F -V -c arduino -p atmega328p -e -P `readlink -m /dev/serial/by-id/{}` -b 115200 -U flash:w:{}".format(tool_config['tools']['avrdude'], ser_id, tf.name)
    foreground_proc(cmdline)

    os.unlink(tf.name)
    return finishMeasurement("atmega328p", em)


def mspexp430f5529(fname):
    em = setupMeasurement("msp-exp430f5529")

    foreground_proc("{} tilib -q \"prog {}\" &".format(tool_config['tools']['mspdebug'], fname))

    return finishMeasurement("msp-exp430f5529", em)


def mspexp430fr5739(fname):
    em = setupMeasurement("msp-exp430fr5739")

    foreground_proc("{} rf2500 -q \"prog {}\" &".format(tool_config['tools']['mspdebug'], fname))

    return finishMeasurement("msp-exp430fr5739", em)


def pic32mx250f128b(fname):
    # Create temporary file and convert to hex file
    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.close()
    foreground_proc("{} -O ihex {} {}".format(tool_config['tools']['pic32_objcopy'], fname, tf.name))

    # Program the PIC and leave power on to run test
    em = setupMeasurement("pic32mx250f128b")
    foreground_proc("{} -p {}".format(tool_config['tools']['pic32prog'], tf.name))

    os.unlink(tf.name)
    return finishMeasurement("pic32mx250f128b", em)


def main():
    arguments = docopt(__doc__)

    logging.basicConfig()

    if arguments['--verbose'] == 1:
        logging.getLogger('').setLevel(logging.INFO)
    elif arguments['--verbose']== 2:
        logging.getLogger('').setLevel(logging.DEBUG)

    loadConfiguration(arguments['--config'])
    loadToolConfiguration(arguments['--tools'])

    if arguments['PLATFORM'] == "stm32f0discovery":
        m = stm32f0discovery(arguments['EXECUTABLE'])
    elif arguments['PLATFORM'] == "stm32vldiscovery":
        m = stm32vldiscovery(arguments['EXECUTABLE'])
    elif arguments['PLATFORM'] == "atmega328p":
        m = atmega328p(arguments['EXECUTABLE'])
    elif arguments['PLATFORM'] == "pic32mx250f128b":
        m = pic32mx250f128b(arguments['EXECUTABLE'])
    elif arguments['PLATFORM'] == "msp-exp430f5529":
        m = mspexp430f5529(arguments['EXECUTABLE'])
    elif arguments['PLATFORM'] == "msp-exp430fr5739":
        m = mspexp430fr5739(arguments['EXECUTABLE'])
    else:
        raise RuntimeError("Unknown platform " + arguments['PLATFORM'])

    print "Energy:          {}J".format(prettyPrint(m.energy))
    print "Time:            {}s".format(prettyPrint(m.time))
#    print "Power:           {}W".format(prettyPrint(m.avg_power))
    print "Average current: {}A".format(prettyPrint(m.avg_current))
    print "Average voltage: {}V".format(prettyPrint(m.avg_voltage))

if __name__ == "__main__":
    main()



#!/usr/bin/python

"""Energy tools

Usage:
    energytool (-m MPOINT)... [options] read SERIAL PIN
    energytool (-m MPOINT)... [options] continuous SERIAL
    energytool (-m MPOINT)... [options] debug SERIAL
    energytool list
    energytool changeserial SERIAL NEWSERIAL

Commands:
    read            This sets up a trigger on the specified PIN and waits for
                    an energy measurement.

    continuous      Continuously read measurements from the specified energy
                    monitor.

    debug           Output some debug data about the instantaneous voltages
                    seen on the ADCs, along with current and voltage.

    list            Show the serial numbers and API version of each connected
                    energy monitor.

    changeserial    Connect to the device specified by SERIAL, and change the
                    serial to NEWSERIAL

Options:
    -m --measurement MPOINT     Specify a measurement point to use (up to 3)
                                can be specified.
    -t --time TIME              Seconds delay between measurements for
                                continuous and debug [default: 0.1]
"""
from docopt import docopt
import pyenergy
from time import sleep
import usb.util

def prettyPrint(v):
    units = ['', 'm', 'u', 'n', 'p']

    for unit in units:
        if v > 1.0:
            return "{: >8.3f} {}".format(v, unit)
        v *= 1000.
    return "{}".format(v)

def display(m):
    print "Energy:          {}J".format(prettyPrint(m.energy))
    print "Time:            {}s".format(prettyPrint(m.time))
    print "Power:           {}W".format(prettyPrint(m.avg_power))
    print "Average current: {}A".format(prettyPrint(m.avg_current))
    print "Average voltage: {}V".format(prettyPrint(m.avg_voltage))


def read(serial, pin, mpoints):
    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    for mp in mpoints:
        em.enableMeasurementPoint(mp)
        em.setTrigger(pin, mp)

    while not em.measurementCompleted():
        sleep(0.1)

    for mp in mpoints:
        m = em.getMeasurement(mp)
        print "Measurement point",mp
        display(m)
        print ""

def continuous(serial, mpoints, delay=0.1):
    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    print "time,",

    for mp in mpoints:
        em.enableMeasurementPoint(mp)
        em.start(mp)
        print "energy_{0}, power_{0}, avg_current_{0}, avg_voltage_{0},".format(mp),
    print ""

    while True:
        first = True
        for mp in mpoints:
            m = em.getMeasurement(mp)

            if first:
                print m.time,
            print "{}, {}, {}, {},".format(m.energy, m.energy/m.time, m.avg_current, m.avg_voltage),
        print ""
        sleep(delay)


def debug(serial, mpoints, delay=0.1):
    em = pyenergy.EnergyMonitor(serial)
    em.connect()

    for mp in mpoints:
        em.enableMeasurementPoint(mp)
        em.start(mp)

    while True:
        for mp in mpoints:
            meas = em.getInstantaneous(mp)
            print "Measurement point:", mp
            em.debugInstantaneous(meas)
        print ""
        sleep(delay)

def list_boards():
    devs = pyenergy.EnergyMonitor.getBoards()

    print "Connected energy monitors:"

    if len(devs) > 0:
        print "    Serial  API"
    else:
        print "    None :("

    for d in devs:
        d.set_configuration()
        v = pyenergy.EnergyMonitor.getVersion(d)

        if v < pyenergy.EnergyMonitor.baseVersion:
            serial = "{} (?)".format(usb.util.get_string(d, 256, 3))
        else:
            serial = pyenergy.EnergyMonitor.getSerial(d)

        print "    {: <8} {}".format(serial, v)

def change_serial(oldser, newser):
    em = pyenergy.EnergyMonitor(oldser)
    em.setSerial(newser)


def main():
    arguments = docopt(__doc__)

    mpoints = map(int, set(arguments['--measurement']))

    if arguments['read']:
        read(arguments['SERIAL'], arguments['PIN'], mpoints)
    elif arguments['debug']:
        debug(arguments['SERIAL'], mpoints, float(arguments['--time']))
    elif arguments['continuous']:
        continuous(arguments['SERIAL'], mpoints, float(arguments['--time']))
    elif arguments['list']:
        list_boards()
    elif arguments['changeserial']:
        change_serial(arguments['SERIAL'], arguments['NEWSERIAL'])

if __name__=="__main__":
    main()



import usb
import usb.core
import usb.util

from struct import *
from copy import copy
from collections import namedtuple

import logging

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())
warning = logger.warning
error = logger.error


# import multiprocessing

Measurement = namedtuple('Measurement', 'energy time peak_power peak_voltage peak_current n_samples avg_voltage avg_current avg_power')

class EnergyMonitor(object):
    MeasurementData = namedtuple('MeasurementData', 'energy_accum elapsed_time peak_power peak_voltage peak_current n_samples avg_current avg_voltage')
    MeasurementData_packing = "=QQLLLLQQ"

    InstantaneousData = namedtuple('InstantaneousData', 'voltage current average_voltage average_current current_time')
    InstantaneousData_packing = "=LLLLQ"

    ADC1 = 0
    ADC2 = 1
    ADC3 = 2

    port_mappings = {1: ["PA2", "PC2"], 2: ["PA3", "PA1"], 3: ["PB1", "PC5"], 4:["PB0", "PC4"]}

    newestVersion = 10
    baseVersion = 10

    def __init__(self, serial="EE00"):
        devs = self.getBoards()

        sdevs = []
        for d in devs:
            d.set_configuration()
            v = self.getVersion(d)
            if v < EnergyMonitor.baseVersion:
                warning("Device attached with old firmware, cannot check if this is desired device")
                continue

            s = self.getSerial(d)

            if s == serial:
                sdevs.append(d)

            # TODO else release config

        devs = sdevs

        if len(devs) > 1:
            warning("More than one device ({}) available with serial {}".format(len(devs), serial))
        if len(devs) == 0:
            raise RuntimeError("Cannot find energy monitor with serial " + serial)

        self.serial = serial
        self.dev = devs[0]

        # Set up default parameters for each measurement point
        defaultparams = {'resistor':1, 'gain':50, 'vref':3}
        self.measurement_params = {i : copy(defaultparams) for i in [1,2,3]}

        # Measurement point 4 is the 'self' measurement point
        self.measurement_params[4] = {'resistor':0.5, 'gain':50, 'vref':3}

        # Equal to tperiod in the firmware
        self.samplePeriod = 500

        self.adcMpoint = [None, None, None]

    @staticmethod
    def getBoards():
        # Find the usb device that corresponds to the serial number
        devs = usb.core.find(idVendor=0xf539, idProduct=0xf539, find_all = True)
        return devs

    # Connect to the device
    def connect(self):
        self.dev.set_configuration()
        self.version = self.getVersion(self)

        if self.version < EnergyMonitor.baseVersion:
            error("Firmware is too old, please update")
        if self.version < EnergyMonitor.newestVersion:
            warning("More recent firmware is available, please update")

    # Get version
    @staticmethod
    def getVersion(dev):
        try:
            dev = dev.dev
        except:
            pass

        try:
            b = dev.ctrl_transfer(0xc1, 12, 0, 0, 4)
        except usb.core.USBError as e:
            if e.errno == 32:
                return 0
            raise
        version = unpack("=L", b)[0]
        return version

    # Get serial
    @staticmethod
    def getSerial(dev):
        try:
            dev = dev.dev
        except:
            pass

        b = dev.ctrl_transfer(0xc1, 13, 0, 0, 4)
        serial = unpack("=4s", b)[0]
        return serial

    # Toggle the LEDs on the device
    def toggleLEDs(self):
        self.dev.ctrl_transfer(0x41, 0, 0, 0, None)

    # Start measuring on m_point
    def start(self, m_point=1):
        self.clearNumberOfRuns()
        self.dev.ctrl_transfer(0x41, 1, int(m_point), 0, None)

    # Stop measuring on m_point
    def stop(self, m_point=1):
        self.dev.ctrl_transfer(0x41, 2, int(m_point), 0, None)

    # Return whether the measurement point is currently taking
    # measurements or not
    def isRunning(self, m_point=1):
        b = self.dev.ctrl_transfer(0xc1, 8, int(m_point), 0, 4)

        running = unpack("=L", b)
        return bool(running[0])

    # This counts the number of end measurement signals caught
    # by the energy monitor.
    def getNumberOfRuns(self, m_point=1):
        b = self.dev.ctrl_transfer(0xc1, 9, int(m_point), 0, 4)

        runs = unpack("=L", b)
        return bool(runs[0])

    # Reset the number of runs counts to 0
    def clearNumberOfRuns(self, m_point=1):
        self.dev.ctrl_transfer(0x41, 10, int(m_point), 0, None)

    # Have we completed a measurement?
    def measurementCompleted(self, m_point = 1):
        runs = self.getNumberOfRuns()
        if runs > 1:
            warning("More than one measurement has completed (expected one)")
        if not self.isRunning(m_point) and runs > 0:
            self.clearNumberOfRuns()
            return True
        return False

    # Set the serial number
    def setSerial(self, ser):
        if len(ser) != 4:
            warning("Serial should be 4 characters.")
            ser = (ser + "0000")[:4]
        self.dev.ctrl_transfer(0x41, 3, ord(ser[0]) | (ord(ser[1])<<8), ord(ser[2]) | (ord(ser[3])<<8), None)

    # Set a particular port as a pin trigger for a measurement point
    #   e.g PA0
    def setTrigger(self, port, m_point=1):

        # TODO check port is of the form PA0
        self.dev.ctrl_transfer(0x41, 4, ord(port[1]) | (m_point<<8), int(port[2]), None)

    # Enable a particular measurement point. There are
    # only 3 ADCs in the device, so only 3 measurement points
    # can be used simultaneously
    def enableMeasurementPoint(self, m_point, adc=None):
        if m_point in self.adcMpoint:
            warning("Tried to enable already enabled measurement point "+str(m_point))
            return
        if self.adcMpoint.count(None) == 0:
            raise RuntimeError("Cannot enable measurement point {}. Maximum of enabled measurement points reached".format(m_point))
        if adc is not None and self.adcMpoint[adc] is not None:
            raise RuntimeError("Cannot enable map measurement point {0} to ADC{1}. ADC{1} already has measurement point {2}".format(m_point, adc, self.adcMpoint[adc]))

        if adc is None:
            # If we want mpoint 1 or 2 and ADC3 is free, prioritise it
            #  becuase ADC3 doesnt work with mpoint 3 or self
            if m_point in [1, 2] and self.adcMpoint[2] is None:
                adc = 2
            else:
                adc = self.adcMpoint.index(None)
        if m_point in [3,4] and adc == 2:
            raise RuntimeError("Measurement point {} cannot be used with ADC3 (the only free ADC)".format(m_point))
        self.adcMpoint[adc] = m_point
        self.dev.ctrl_transfer(0x41, 7, int(m_point), adc, None)

    # Disable a particular measurement point, so that the
    # ADC could potentially be used with a different one
    def disableMeasurementPoint(self, m_point):
        if m_point not in self.adcMpoint:
            warning("Tried to disable already disabled measurement point "+str(m_point))
            return
        adc = self.adcMpoint.index(m_point)
        self.adcMpoint[adc] = None
        # TODO: perhaps a control transfer to actually disable the mpoint

    def convertData(self, md, resistor, gain, vref, samplePeriod):
        en = float(vref)**2 / gain / resistor / 4096**2 * 2 * samplePeriod * 2 / 168000000. * md.energy_accum * 2
        el = md.elapsed_time * 2. / 168000000 * 2
        pp = float(vref)**2 / gain / resistor / 4096**2 * md.peak_power * 2
        pv = float(vref) / 4096. * md.peak_voltage * 2
        pi = float(vref) / gain / resistor / 4096. * md.peak_current
        av = float(vref) / 4096. * md.avg_voltage / md.n_samples * 2
        ai = float(vref) / gain / resistor / 4096. * md.avg_current / md.n_samples
        ap = en/el

        m = Measurement(en, el, pp, pv, pi, md.n_samples, av, ai, ap)

        return m

    def getMeasurement(self, m_point=1):
        b = self.dev.ctrl_transfer(0xc1, 6, int(m_point), 0, calcsize(EnergyMonitor.MeasurementData_packing))
        u = EnergyMonitor.MeasurementData._make(unpack(EnergyMonitor.MeasurementData_packing, b))

        return self.convertData(u, samplePeriod=self.samplePeriod, **self.measurement_params[m_point])

    # get an instantaneous measurement of voltage and current (debugging)
    def getInstantaneous(self, m_point=1):
        b = self.dev.ctrl_transfer(0xc1, 11, int(m_point), 0, calcsize(EnergyMonitor.InstantaneousData_packing))
        args = list(unpack(EnergyMonitor.InstantaneousData_packing, b))
        args.append(m_point)
        return args

    # Convert and display instantaneous measurement
    def debugInstantaneous(self, v):
        mp = v[5]
        resistor = self.measurement_params[mp]['resistor']
        gain = self.measurement_params[mp]['gain']
        vref = self.measurement_params[mp]['vref']

        print "Timestamp:", v[4] * 2. / 168000000 * 2
        print "Current:  Raw={:4d}  Voltage@{}={:1.3f}V  Res Vdrop={:1.5f}V  Current={:1.5f}A".format(mp,
            EnergyMonitor.port_mappings[mp][1],
            v[3]/4096.*vref,
            float(vref) / gain / 4096. * v[3],
            float(vref) / gain / resistor / 4096. * v[3])
        print "Voltage:  Raw={:4d}  Voltage@{}={:1.3f}V                      Voltage={:1.5f}V".format(v[2],
            EnergyMonitor.port_mappings[mp][0],
            v[2]/4096.*vref,
            float(vref) / 4096. * v[2] * 2)
        print ""

    def disconnect(self):
        pass



if __name__ == "__main__":
    from time import sleep

    em = EnergyMonitor("EE00")
    em.connect()

    em.toggleLEDs()

    em.enableMeasurementPoint(1)
    em.enableMeasurementPoint(2)

    em.setTrigger("PA0", 1)
    em.setTrigger("PA0", 2)

    print "*** Press the blue button to make a measurement"

    while True:
        while not em.measurementCompleted(): sleep(0.1)
        print em.getMeasurement(1)
        print em.getMeasurement(2)



import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "pyenergy",
    version = "0.1rc2",
    author = "James Pallister",
    author_email = "james.pallister@bristol.ac.uk",
    description = ("An interface to the MAGEEC energy monitor boards"),
    license = "GPLv3",
    keywords = "energy monitor MAGEEC",
    # url = "http://packages.python.org/an_example_pypi_project",
    package_dir={'':'src'},
    packages=['pyenergy', 'platformrun'],
    long_description=read('README.rst'),
    install_requires=['pyusb>=1.0.0b1', 'docopt>=0.6.1'],
    zip_safe=True,
    entry_points = {
        'console_scripts': [
            'platformrun = platformrun:main',
            'platformrun-detect = platformrun.detect:main',
            'energytool = pyenergy.tools:main',
        ],
        # 'gui_scripts': [
        #     'interactive_graph = pyenergy.interactive_graph:main'
        # ]
    }
)
